<?php

/**
 * Webkul Software.
 *
 * @category  Webkul
 *
 * @author    Webkul Software Private Limited
 * @copyright Copyright (c) 2010-2017 Webkul Software Private Limited (https://webkul.com)
 * @license   https://store.webkul.com/license.html
 */

namespace Webkul\SalesforceConnector\Controller\Adminhtml\Contact;

use Webkul\SalesforceConnector\Helper\Customer;

class Delete extends \Magento\Backend\App\Action
{
    /** @var \Magento\Framework\View\Result\PageFactory */
    protected $_resultPageFactory;
    protected $_helper;
    /**
     * Massactions filter.
     *
     * @var Filter
     */
    protected $_filter;

    /**
     * @var CollectionFactory
     */
    protected $_collectionFactory;

    /**
     * @param \Magento\Backend\App\Action\Context        $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Ui\Component\MassAction\Filter $filter,
        \Magento\Customer\Model\ResourceModel\Address\CollectionFactory $collectionFactory,
        Customer $helper
    ) {
        parent::__construct($context);
        $this->_resultPageFactory = $resultPageFactory;
        $this->_helper = $helper;
        $this->_filter = $filter;
        $this->_collectionFactory = $collectionFactory;
    }

    public function execute()
    {
        $id = $this->getRequest()->getParam('entity_id');
        $recordDeleted = 0;
        if ($id) {
            $this->_helper->deleteContactMapping($id);
            ++$recordDeleted;
        } else {
            $ids = $this->getRequest()->getParam('selected');

            if ($ids) {
                if ($ids && count($ids) > 0) {
                    foreach ($ids as $id) {
                        $this->_helper->deleteContactMapping($id);
                        ++$recordDeleted;
                    }
                }
            } else {
                $excludedIds = $this->getRequest()->getParam('excluded');
                $collection = $this->_collectionFactory->create();
                $allCustomerIds = array();
                foreach ($collection->getItems() as $item) {
                    array_push($allCustomerIds, $item->getID());
                }
                if ($excludedIds == 'false') {

                    if ($allCustomerIds && count($allCustomerIds) > 0) {
                        foreach ($allCustomerIds as $id) {
                            $this->_helper->deleteContactMapping($id);
                            ++$recordDeleted;
                        }
                    }
                } else {
                    $ids = array_diff($allCustomerIds, $excludedIds);

                    if ($ids && count($ids) > 0) {
                        foreach ($ids as $id) {
                            $this->_helper->deleteContactMapping($id);
                            ++$recordDeleted;
                        }
                    }
                }
            }
        }
        $this->messageManager->addSuccess(
            __('A total of %1 record(s) have been deleted.', $recordDeleted)
        );
        $resultRedirect = $this->resultRedirectFactory->create();
        $resultRedirect->setPath('sfconnector/contact/index/');

        return $resultRedirect;
    }
}
