<?php

/**
 * * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Webkul\SalesforceConnector\Controller\Adminhtml\System\Config\Connection;

use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\App\PageCache\Version;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Cache\Frontend\Pool;

class Connect extends \Webkul\SalesforceConnector\Controller\Adminhtml\System\Config\Connection
{
    /**
     * @var JsonFactory
     */
    protected $_json;
    protected $writerInterface;
    protected $_curl;
    protected $cacheTypeList;
    protected $_scopeconfig;
    protected $cacheFrontendPool;
    /**
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Webkul\Eshopsync\Helper\Connection $connection
     * @param \Webkul\Eshopsync\Helper\Data $helper
     * @param \Magento\Config\Model\ResourceModel\Config $resourceConfig
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\Json\Helper\Data $jsonHelper,
        \Magento\Framework\App\Config\Storage\WriterInterface $writerInterface,
        TypeListInterface $cacheTypeList,
        Pool $cacheFrontendPool,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\HTTP\Client\Curl $curl
    ) {

        parent::__construct($context);
        $this->writerInterface = $writerInterface;
        $this->_json = $jsonHelper;
        $this->_curl = $curl;
        $this->_scopeconfig = $scopeConfig;
        $this->cacheTypeList = $cacheTypeList;
        $this->cacheFrontendPool = $cacheFrontendPool;
    }

    public function execute()
    {   
        $response = [];
        $post = $this->getRequest()->getPostValue();
        $code = $post['code'];
        $instance = $post['instance'];
        if ($instance == 'production') {
            $url = 'https://login.salesforce.com/';
        } else {
            $url = 'https://test.salesforce.com/';
        }
        $curlUrl = $url.
                    'services/oauth2/token?grant_type=authorization_code'.
                    '&client_id=3MVG9ZL0ppGP5UrDmNAiEGMQGPYAYCNk.hb1xNVmqWoDi21Ug18eq6ucsSD1ZanfepgDlh0BNUF3gNjwjZaEM&'.
                    'client_secret=3093543145986445979&'.
                    'redirect_uri='.urlencode("https://eshopsync.com/connector/auth.php").
                    '&code='.$code;
        try {
            $this->_curl->post($curlUrl, []);
            $result = $this->_curl->getBody();
            $resultData = $this->_json->jsonDecode($result);
            if (isset($resultData) && isset($resultData['access_token'])) {
                $c_url = $resultData['instance_url'] . '/services/data/v46.0/sobjects/Account/describe/';
                $headers = [
                    'Authorization' => $resultData['token_type'] . ' ' . $resultData['access_token'],
                    'Cache-Control' => 'no-cache',
                ];
                $ispersonEnabledOrg = false;
                $this->_curl->setHeaders($headers);
                $this->_curl->get($c_url);
                $resp = $this->_curl->getBody();
                $resp = $this->_json->jsonDecode($resp);
                if (isset($resp['fields']) && !empty($resp['fields'])) {
                    $fields = $resp['fields'];
                    $err = false;
                    foreach ($fields as $field) {
                        if($field['name'] == 'IsPersonAccount'){
                            $ispersonEnabledOrg = true;
                        }
                    }
                    $orgtype = $this->_scopeconfig->getValue('sfConnect/general/orgtype');
                    if($orgtype == 1 && $ispersonEnabledOrg==true){
                        $this->messageManager->addError(__("You seem to be using business edition of this extension, but you are connecting Person Enabled Org of Salesforce. Please connect with Business Enabled Org."));
                        $response['con_status'] = 0;
                        $err = true;
                    }
                    if($orgtype == 0 && $ispersonEnabledOrg==false){
                        $this->messageManager->addError(__("You seem to be using the Person edition of this extension, but you are connecting Business Org of Salesforce. Please connect with Person Enabled Org."));
                        $response['con_status'] = 0;
                        $err = true;
                    }
                    if(!$err){
                        $this->writerInterface->save('sfConnect/general/restconfig', $result);
                        $this->writerInterface->save('sfConnect/general/connection_status', '1');
                        $response['con_status'] = 1;
                        if ($instance == 'production') {
                            $this->writerInterface->save('sfConnect/general/enviornment', 'P');
                        } else {
                            $this->writerInterface->save('sfConnect/general/enviornment', 'S');
                        }
                        $this->messageManager->addSuccess(__('Connection created successfully'));
                    }
                }
                
            } else {          
                $response['con_status'] = 0;
            }
           
            $this->cacheTypeList->cleanType('config');
            foreach ($this->cacheFrontendPool as $cacheFrontend) {
                $cacheFrontend->getBackend()->clean();
            }
            
        } catch (\Exception $e) {
            $this->messageManager->addError(__('Error while code fetch'));
        }
        
        $this->getResponse()->setHeader('Content-type', 'application/json');
        $this->getResponse()->setBody($response['con_status']);
    }

    /*
     * Check permission via ACL resource
     */
}
