<?php

/**
 * Webkul Software.
 *
 * @category  Webkul
 *
 * @author    Webkul Software Private Limited
 * @copyright Copyright (c) 2010-2017 Webkul Software Private Limited (https://webkul.com)
 * @license   https://store.webkul.com/license.html
 */

namespace Webkul\SalesforceConnector\Helper;

use Magento\Framework\App\Helper\AbstractHelper;

/**
 * Eshopsync category helper.
 */
class Order extends AbstractHelper
{
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;
    protected $_directory;
    protected $_eventManager;
    protected $_json;
    protected $_resourceConfig;
    protected $_image;
    protected $_salesItem;
    protected $_salesOrder;
    protected $_catalogProduct;
    protected $_eshopsyncOrder;
    protected $_eshopsynProduct;
    protected $_helperData;
    protected $_helperConnection;
    protected $_helperCustomer;
    protected $_eshopsyncCustomer;
    protected $_helperProduct;

    /**
     * @param Magento\Framework\App\Helper\Context      $context
     * @param Magento\Store\Model\StoreManagerInterface $storeManager
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\App\Filesystem\DirectoryList $directory,
        \Magento\Framework\Event\Manager $eventManager,
        \Magento\Framework\Json\Helper\Data $jsonHelper,
        \Magento\Config\Model\ResourceModel\Config $resourceConfig,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Model\ProductFactory $catalogProduct,
        \Magento\Catalog\Helper\Image $image,
        \Magento\Sales\Model\Order\ItemFactory $salesItem,
        \Magento\Sales\Model\OrderFactory $salesOrder,
        \Webkul\SalesforceConnector\Model\OrderFactory $eshopsyncOrder,
        \Webkul\SalesforceConnector\Model\OpportunityFactory $eshopsyncOpportunity,
        \Webkul\SalesforceConnector\Model\ProductFactory $eshopsynProduct,
        \Webkul\SalesforceConnector\Helper\Data $helperData,
        \Webkul\SalesforceConnector\Helper\Connection $helperConnection,
        \Webkul\SalesforceConnector\Helper\Customer $helperCustomer,
        \Webkul\SalesforceConnector\Model\CustomerFactory $eshopsyncCustomer,
        \Webkul\SalesforceConnector\Helper\Product $helperProduct,
        \Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable $catalogProductTypeConfigurable,
        \Magento\Bundle\Model\Product\Type $catalogProductTypeBundle
    ) {
        $this->_storeManager = $storeManager;
        $this->_directory = $directory;
        $this->_eventManager = $eventManager;
        $this->_json = $jsonHelper;
        $this->_resourceConfig = $resourceConfig;
        $this->_catalogProduct = $catalogProduct;
        $this->_image = $image;
        $this->_salesItem = $salesItem;
        $this->_salesOrder = $salesOrder;
        $this->_eshopsyncOrder = $eshopsyncOrder;
        $this->_eshopsyncOpportunity = $eshopsyncOpportunity;
        $this->_eshopsynProduct = $eshopsynProduct;
        $this->_helperData = $helperData;
        $this->_helperConnection = $helperConnection;
        $this->_helperCustomer = $helperCustomer;
        $this->_eshopsyncCustomer = $eshopsyncCustomer;
        $this->_helperProduct = $helperProduct;
        $this->_catalogProductTypeConfigurable = $catalogProductTypeConfigurable;
        $this->_catalogProductTypeBundle = $catalogProductTypeBundle;
        $this->_scopConfig = $context->getScopeConfig();
        parent::__construct($context);
    }

    public function orderMapping($data, $error = 0, $status = 'Export')
    {
        $model = $this->_eshopsyncOrder->create();
        $collection = $model->getCollection()->addFieldToFilter('magento_id', $data['magento_id'])->getData();
        if (count($collection) == 1) {
            $data['entity_id'] = $collection[0]['entity_id'];
        } elseif (count($collection) > 1) {
            foreach ($collection as $row) {
                $model->setId($row['entity_id'])->delete();
            }
        }
        $model->setData($data);
        $model->save();
    }

    public function opportunityMapping($data, $error = 0, $status = 'Export')
    {
        $model = $this->_eshopsyncOpportunity->create();
        $collection = $model->getCollection()->addFieldToFilter('magento_id', $data['magento_id'])->getData();
        if (count($collection) == 1) {
            $data['entity_id'] = $collection[0]['entity_id'];
        } elseif (count($collection) > 1) {
            foreach ($collection as $row) {
                $model->setId($row['entity_id'])->delete();
            }
        }
        $model->setData($data);
        $model->save();
    }

    public function createOrderReqArr($thisOrder, $orderId, &$reqArr, &$accountIdArr)
    {
        $accountId = $this->getSalesforceAccountId($thisOrder, $reqArr);
        if ($accountId) {
            $accountIdArr[$orderId] = $accountId;
            $orderSync = $this->_helperData->getConfigValue('orderCon', 'orderSync');
            $oppSync = $this->_helperData->getConfigValue('orderCon', 'oppSync');
            if ($orderSync) {
                if ($this->createSalesforceOrder($thisOrder, $accountId, $reqArr) === 0) {
                    return 0;
                }
                $this->createSalesforceOrderLine($thisOrder, $reqArr);
            }
            if ($oppSync) {
                $this->createSalesforceOpportunity($thisOrder, $accountId, $reqArr);
                $this->createSalesforceOpportunityLine($thisOrder, $reqArr);
            }
        }
    }

    public function syncBulkOrders($ids)
    {
        $success = 0;
        $error = 0;
        $skip = 0;
        try {
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $productCollection = $objectManager->create(\Magento\Sales\Model\ResourceModel\Order\Collection::class);
            $collection = $productCollection->addFieldToFilter('entity_id', ['in' => $ids]);
            $requestArray = [];
            $oldReqArr = [];
            $accountIdArr = $reqArr = [];

            foreach ($collection as $order) {
                $oldReqArr = $reqArr;
                $thisOrder = $order->load($order['entity_id']);
                if ($this->createOrderReqArr($thisOrder, $order['entity_id'], $reqArr, $accountIdArr) === 0) {
                    return [
                        'success' => 0,
                        'error' => 'PriceBook not Selected! Please check Config'
                    ];
                }
                if (count($reqArr) > 25) {
                    $requestArray[] = $oldReqArr;
                    $reqArr = [];
                    $this->createOrderReqArr($thisOrder, $order['entity_id'], $reqArr, $accountIdArr);
                }
            }
            if (count($reqArr) > 0) {
                $requestArray[] = $reqArr;
            }

            if (count($requestArray) > 0) {
                foreach ($requestArray as $req) {
                    $response = $this->_helperConnection->compositeUpsertSObject(['compositeRequest' => $req]);
                    foreach ($response['compositeResponse'] as $result) {
                        if (isset($result['body']['success'])) {
                            $success = 1;
                        } else if (isset($result['body'][0]['message'])) {
                            $error = 1;
                            \Main\Support\Root\Logger::instance('salesforce_debug_')
                                ->log(print_r($result['body'], true));
                        } else {
                            $skip = 1;
                        }

                        $mappingData = [];
                        if (strpos($result['referenceId'], 'order_') !== false) {
                            $orderId = ltrim($result['referenceId'], 'order_');
                            if (isset($result['body']['success'])) {
                                $mappingData = [
                                    'magento_id' => $orderId,
                                    'sync_status' => 1,
                                    'account_id' => $accountIdArr[$orderId],
                                    'sforce_id' => $result['body']['id'],
                                ];
                            } elseif (isset($result['body'][0]['message'])) {
                                $mappingData = [
                                    'magento_id' => $orderId,
                                    'sync_status' => 0,
                                    'account_id' => $accountIdArr[$orderId],
                                    'sforce_id' => 0,
                                    'error' => $result['body'][0]['message'],
                                ];
                            }
                            if (count($mappingData) > 0) {
                                $this->orderMapping($mappingData);
                            }
                        }
                        if (strpos($result['referenceId'], 'opportunity_') !== false) {
                            $orderId = ltrim($result['referenceId'], 'opportunity_');
                            if (isset($result['body']['success'])) {
                                $sfOrderID = $result['body']['id'];
                                $mappingData = [
                                    'magento_id' => $orderId,
                                    'sync_status' => 1,
                                    'account_id' => $accountIdArr[$orderId],
                                    'sforce_id' => $result['body']['id'],
                                ];
                            } elseif (isset($result['body'][0]['message'])) {
                                $mappingData = [
                                    'magento_id' => $orderId,
                                    'sync_status' => 0,
                                    'account_id' => $accountIdArr[$orderId],
                                    'sforce_id' => '',
                                    'error' => $result['body'][0]['message'],
                                ];
                            }
                            if (count($mappingData) > 0) {
                                $this->opportunityMapping($mappingData);
                            }
                        }
                    }
                }
            }
            
            return [
                'success' => $success > 0,
                'error' => $error > 0,
                'skip' => $skip > 0
            ];
        } catch (\Exception $e) {
            $this->_helperData->eshopsyncLog($e->getMessage());
            return false;
        }
    }

    public function syncSpecificOrder($orderId, $order = null, $autoOpp = null)
    {
        $reqArr = [];
        $sfOrderID = false;
        if (!empty($order)) {
            $thisOrder = $order;
        } else {
            $thisOrder = $this->_salesOrder->create()->load($orderId);
        }
        $accountId = $this->getSalesforceAccountId($thisOrder, $reqArr);

        if ($accountId) {

            $orderSync = $this->_helperData->getConfigValue('orderCon', 'orderSync');
            $oppSync = $this->_helperData->getConfigValue('orderCon', 'oppSync');
            if ($orderSync) {
                if($this->createSalesforceOrder($thisOrder, $accountId, $reqArr)===0)
                {   
                    
                    return ["error" => "PriceBook not Selected! Please check Config."];
                }
                $this->createSalesforceOrderLine($thisOrder, $reqArr);
            }
            else{
                return ["error" => "Export order functionality needs to be enabled from Configuration."];
            }
            if ($oppSync) {
                if (!empty($autoOpp)) {
                    if ($autoOpp) {
                        $this->createSalesforceOpportunity($thisOrder, $accountId, $reqArr);
                        $this->createSalesforceOpportunityLine($thisOrder, $reqArr);
                    }
                } else {
                    $this->createSalesforceOpportunity($thisOrder, $accountId, $reqArr);
                    $this->createSalesforceOpportunityLine($thisOrder, $reqArr);
                }
            }

            if (count($reqArr) > 0) {
                $response = $this->_helperConnection->compositeUpsertSObject(['compositeRequest' => $reqArr]);
                

                foreach ($response['compositeResponse'] as $result) {
                    $mappingData = [];
                    if (strpos($result['referenceId'], 'Guest_') !== false) {

                        if (isset($result['body']['success'])) {
                            $accountId = $result['body']['id'];
                        } elseif (isset($result['body'][0]['message'])) {

                            $mappingData = [
                                'magento_id' => $orderId,
                                'sync_status' => 0,
                                'account_id' => $accountId,
                                'sforce_id' => '',
                                'error' => $result['body'][0]['message'],
                            ];
                        }
                        if (count($mappingData) > 0) {
                            $this->orderMapping($mappingData);
                            break;
                        }
                    }
                    if (strpos($result['referenceId'], 'order_') !== false) {
                        if (isset($result['body']['success'])) {
                            $sfOrderID = $result['body']['id'];
                            $mappingData = [
                                'magento_id' => $orderId,
                                'sync_status' => 1,
                                'account_id' => $accountId,
                                'sforce_id' => $result['body']['id'],
                            ];
                        } elseif (isset($result['body'][0]['message'])) {
                            $mappingData = [
                                'magento_id' => $orderId,
                                'sync_status' => 0,
                                'account_id' => $accountId,
                                'sforce_id' => '',
                                'error' => $result['body'][0]['message'],
                            ];
                        }
                        if (count($mappingData) > 0) {
                            $this->orderMapping($mappingData);
                        }
                    }
                    if (strpos($result['referenceId'], 'opportunity_') !== false) {
                        if (isset($result['body']['success'])) {
                            $sfOrderID = $result['body']['id'];
                            $mappingData = [
                                'magento_id' => $orderId,
                                'sync_status' => 1,
                                'account_id' => $accountId,
                                'sforce_id' => $result['body']['id'],
                            ];
                        } elseif (isset($result['body'][0]['message'])) {
                            $mappingData = [
                                'magento_id' => $orderId,
                                'sync_status' => 0,
                                'account_id' => $accountId,
                                'sforce_id' => '',
                                'error' => $result['body'][0]['message'],
                            ];
                        }
                        if (count($mappingData) > 0) {
                            $this->opportunityMapping($mappingData);
                        }
                    }
                }
            }
        } else {
            $guestError = '';
            if ($thisOrder->getCustomerIsGuest() == 1) {
                $accountDetail = $this->_scopConfig
                    ->getValue(
                        'sfConnect/accCon/guestAcc',
                        \Magento\Store\Model\ScopeInterface::SCOPE_STORE
                    );
                
                if(isset($accountDetail) && ($accountDetail != '')){
                    if (!($this->_json->jsonDecode($accountDetail)['id'])) {
                        $guestError = 'Guest Accountid missing from config';
                    }
                }
                else{
                    $guestError = 'Guest Accountid missing from config';
                }
            }
            $error = 'Order Export Error, Order Id ' . $thisOrder->getIncrementId() . ' Reason Customer Account not found ' . $guestError;
            $errorMappingData = [
                'magento_id' => $orderId,
                'sync_status' => 0,
                'account_id' => 0,
                'sforce_id' => 0,
                'error' => $error,
            ];
            $this->orderMapping($errorMappingData);
            $this->_helperData->eshopsyncLog($error);
        }

        return $sfOrderID;
    }

    public function getGuestAsAccount($billing_address, $shipping_address)
    {
        $sObject = new \stdclass();
        if ($billing_address) {
            $sObject->Phone = $billing_address->getTelephone();
            $streets = implode(',', $billing_address->getStreet());
            $sObject->BillingStreet = $streets;
            $sObject->BillingCity = $billing_address->getCity();
            $sObject->BillingPostalCode = $billing_address->getPostcode();
            $sObject->BillingState = $billing_address->getRegion();
            $sObject->BillingCountry = $billing_address->getCountryId();
        }
        if ($shipping_address) {
            $streets = implode(',', $shipping_address->getStreet());
            $sObject->ShippingStreet = $streets;
            $sObject->ShippingCity = $shipping_address->getCity();
            $sObject->ShippingPostalCode = $shipping_address->getPostcode();
            $sObject->ShippingState = $shipping_address->getRegion();
            $sObject->ShippingCountry = $shipping_address->getCountryId();
        }

        // Fields for person account

        $recordType = $this->_helperData->getConfigValue('accCon', 'recordAcc');
        $sObject->RecordTypeId = $this->_json->jsonDecode($recordType)['id'];
        $sObject->FirstName = $billing_address->getFirstname();
        $sObject->LastName = $billing_address->getLastName() ?? $billing_address->getFirstname();

        $sObject->PersonEmail = $billing_address->getEmail();
        $sObject->PersonMobilePhone = $billing_address->getTelephone();
        $sObject->PersonMailingCity = $sObject->BillingCity;
        $sObject->PersonMailingCountry = $sObject->BillingCountry;
        $sObject->PersonMailingPostalCode = $sObject->BillingPostalCode;
        $sObject->PersonMailingState = $sObject->BillingState;
        $sObject->PersonMailingStreet = $sObject->BillingStreet;
        return $sObject;
    }

    public function getSalesforceAccountId($thisOrder, &$reqArr)
    {
        $orgType = $this->_helperData->getConfigValue('general', 'orgtype');
        $accountId = false;
        try {
            $billing_address = $thisOrder->getBillingAddress();
            $shipping_address = $thisOrder->getShippingAddress();

            $customerId = $thisOrder->getCustomerId();
            if ($thisOrder->getCustomerIsGuest() == 1 || empty($customerId)) {

                $increment_id = $thisOrder->getIncrementId();
                if ($orgType) {

                    $accountDetail = $this->_scopConfig
                        ->getValue(
                            'sfConnect/accCon/guestAcc',
                            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
                        );
                    $accountId = $this->_json->jsonDecode($accountDetail)['id'];
                    if (!$accountId) {
                        $this->messageManager->addError(__('Guest Accountid missing'));

                        return $accountId;
                    }

                    $guestDuplicacy = $this->_scopConfig
                        ->getValue(
                            'sfConnect/accCon/guestAccDuplicacy',
                            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
                        );


                    $sObject = new \stdclass();
                    $sObject->FirstName = $billing_address->getFirstname();
                    $sObject->LastName = $billing_address->getLastname();
                    $sObject->AccountId = $accountId;
                    $sObject->Email = $billing_address->getEmail();
                    $streets = implode(',', $billing_address->getStreet());
                    $sObject->MailingStreet = $streets;
                    $sObject->MailingCity = $billing_address->getCity();
                    $sObject->MailingPostalCode = $billing_address->getPostcode();
                    $sObject->Phone = $billing_address->getTelephone();
                    $sObject->MailingCountry = $billing_address->getCountryId();
                    $sObject->MailingState = $billing_address->getRegion();
                    $address_id = $billing_address->getEntityId();

                    if ($guestDuplicacy) {
                        $reqArr[] = [
                            'method' => 'PATCH',
                            'referenceId' => 'Guest_' . $address_id,
                            'url' => '/services/data/v46.0/sobjects/Contact/webkul_es_mage__Magento_Contact_ID__c/' . $address_id,
                            'body' => $sObject,
                        ];
                    } else {
                        $email = $billing_address->getEmail();
                        $guestAcc = $this->_helperConnection->getSObject("SELECT Id FROM Contact WHERE Email = '" . $email . "'");

                        if (!count($guestAcc) > 0) {

                            $reqArr[] = [
                                'method' => 'PATCH',
                                'referenceId' => 'Guest_' . $address_id,
                                'url' => '/services/data/v46.0/sobjects/Contact/webkul_es_mage__Magento_Contact_ID__c/' . $address_id,
                                'body' => $sObject,
                            ];
                        }
                    }

                   
                } else {
                    // check same account settings:
                    $singleAccountEnable = $this->_scopConfig
                        ->getValue(
                            'sfConnect/accCon/accountEmailCheck',
                            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
                        );
                    if ($singleAccountEnable) {
                        $checkBYEmail = '';
                        if (!empty($billing_address)) {
                            $checkBYEmail = $billing_address->getEmail();
                        } elseif (!empty($shipping_address)) {
                            $checkBYEmail = $shipping_address->getEmail();
                        }
                        if (!empty($checkBYEmail)) {
                            $accountId = $this->_helperCustomer->getExisitingAccountByEmail($checkBYEmail);
                        }
                    }
                    if (empty($accountId)) {
                        $address_id = !empty($billing_address) ? $billing_address->getEntityId() : $shipping_address->getEntityId();
                        $mapObject = $this->getGuestAsAccount($billing_address, $shipping_address);
                        $reqArr[] = [
                            'method' => 'POST',
                            'referenceId' => 'Guest_' . $address_id,
                            'url' => '/services/data/v46.0/sobjects/Account/',
                            'body' => $mapObject,
                        ];
                        $accountId = '@{Guest_' . $address_id . '.id}';
                    }
                }
            }
            if ($customerId !== null && $customerId > 0) {
                $data = $this->_helperData->fetchMappingDetails($this->_eshopsyncCustomer, $customerId);
                if ($data) {
                    $accountId = $data[0]['sforce_id'];
                } else {
                    $accountId = $this->_helperCustomer->exportSpecificCustomer($customerId);
                }
            }
        } catch (\Exception $e) {
            $this->_helperData->eshopsyncLog($e->getMessage());
        }
        return $accountId;
    }

    public function syncAddressAsContactGuest($address, $account_id, $order_id)
    {
        $sforce_id = false;
        $sObject = new \stdclass();
        $sObject->FirstName = $address->getFirstname();
        $sObject->LastName = $address->getLastname();
        $sObject->AccountId = $account_id;
        $sObject->Email = $address->getEmail();
        $streets = implode(',', $address->getStreet());
        $sObject->MailingStreet = $streets;
        $sObject->MailingCity = $address->getCity();
        $sObject->MailingPostalCode = $address->getPostcode();
        $sObject->Phone = $address->getTelephone();
        $sObject->MailingCountry = $address->getCountryId();
        $sObject->MailingState = $address->getRegion();
        $sObject->webkul_es_mage__Magento_Order_ID__c = $order_id;

        return $sObject;
    }

    public function syncOtherAddressAsContactGuest(&$sObject, $address)
    {
        $streets = implode(',', $address->getStreet());
        $sObject->OtherStreet = $streets;
        $sObject->OtherCity = $address->getCity();
        $sObject->OtherPostalCode = $address->getPostcode();
        $sObject->OtherPhone = $address->getTelephone();
        $sObject->OtherCountry = $address->getCountryId();
        $sObject->OtherState = $address->getRegion();
    }

    public function createSalesforceOrder($thisOrder, $accountId, &$reqArr)
    {
        $incrementId = $thisOrder->getIncrementId();
        $orderRecords = new \stdclass();
        $orderRecords->AccountId = $accountId;
        $orderRecords->Status = 'draft';
        $orderRecords->EffectiveDate = date("Y-m-d\TH:i:s\Z", strtotime($thisOrder->getCreatedAt()));
        /*setting default pricebook*/
        $priceBook = $this->_helperData->getConfigValue('catPdt', 'pricebook');
        if(!(isset($priceBook) && $priceBook)){
            return 0;
        }
        $orderRecords->Pricebook2Id = $this->_json->jsonDecode($priceBook)['id'];
        /*billing information*/
        $billingAddress = $thisOrder->getBillingAddress();
        $orderRecords->BillingCity = $billingAddress->getCity();
        $orderRecords->BillingCountry = $billingAddress->getCountryId();
        $orderRecords->BillingPostalCode = $billingAddress->getPostcode();
        $orderRecords->BillingState = $billingAddress->getRegion();
        $streets = implode(',', $billingAddress->getStreet());
        $orderRecords->BillingStreet = $streets;

        /*shipping information*/
        $shippingAddress = $thisOrder->getShippingAddress();
        if ($shippingAddress) {
            $orderRecords->ShippingCity = $shippingAddress->getCity();
            $orderRecords->ShippingCountry = $shippingAddress->getCountryId();
            $orderRecords->ShippingPostalCode = $shippingAddress->getPostcode();
            $orderRecords->ShippingState = $shippingAddress->getRegion();
            $streets = implode(',', $shippingAddress->getStreet());
            $orderRecords->ShippingStreet = $streets;
        }
        $orderRecords->webkul_es_mage__Magento_Order_Status__c = $thisOrder->getStatus();
        $orderRecords->webkul_es_mage__Magento_Payment_Method__c = $thisOrder->getPayment()->getMethodInstance()->getTitle();
        $orderRecords->webkul_es_mage__Magento_Shipment_Method__c = $thisOrder->getShippingDescription();
        $orderRecords->webkul_es_mage__Magento_Shipping_Handling__c = $thisOrder->getBaseShippingInclTax();
        $orderRecords->webkul_es_mage__Magento_Order_Total__c = !empty($thisOrder->getTotalPaid()) ? $thisOrder->getTotalPaid() : $thisOrder->getTotalDue();
        $orderRecords->webkul_es_mage__Magento_Order_Tax__c = $thisOrder->getTaxAmount();
        $orderRecords->webkul_es_mage__Magento_Order_Discount__c = $thisOrder->getDiscountAmount();

        $ordFieldMap = $this->_helperData->getConfigValue('ord', 'orderFieldMap');
        if ($ordFieldMap) {
            $ordFieldMap = $this->_json->jsonDecode($ordFieldMap);

            if(is_array($ordFieldMap) && count($ordFieldMap) > 0){
              
                foreach ($ordFieldMap as $field) {

                    $magentoField = $field['magentoField'];
                    $fieldCall = 'get' . $magentoField;
                    $sfField = $field['saleForceField'];
                    $orderRecords->$sfField = $thisOrder->{$fieldCall}();
                }
            }
        }

        try {
            $reqArr[] = [
                'method' => 'PATCH',
                'referenceId' => 'order_' . $thisOrder->getId(),
                'url' => '/services/data/v46.0/sobjects/order/webkul_es_mage__Magento_Order_ID__c/' . $thisOrder->getId(),
                'body' => $orderRecords,
            ];
        } catch (\Exception $e) {
            $error = 'Order Export Error, Order Id ' . $incrementId . ' Reason >>' . $e->getMessage();
            $this->_helperData->eshopsyncLog($error);
        }
    }
    public function createSalesforceOpportunity($thisOrder, $accountId, &$reqArr)
    {
        $incrementId = $thisOrder->getIncrementId();
        $opportunityRecords = new \stdclass();
        $opportunityRecords->AccountId = $accountId;
        $opportunityRecords->StageName = 'prospecting';
        $opportunityRecords->CloseDate = date("Y-m-d\TH:i:s\Z", strtotime($thisOrder->getCreatedAt()));
        /*setting default pricebook*/
        $priceBook = $this->_helperData->getConfigValue('catPdt', 'pricebook');
        if(isset($priceBook) && $priceBook)
        $opportunityRecords->Pricebook2Id = $this->_json->jsonDecode($priceBook)['id'];
        /*billing information*/
        $opportunityRecords->Amount = !empty($thisOrder->getTotalPaid()) ? $thisOrder->getTotalPaid() : $thisOrder->getTotalDue();

        try {
            $reqArr[] = [
                'method' => 'PATCH',
                'referenceId' => 'opportunity_' . $thisOrder->getId(),
                'url' => '/services/data/v46.0/sobjects/opportunity/Name/' . $_SERVER['SERVER_NAME'] . '_' . $thisOrder->getId(),
                'body' => $opportunityRecords,
            ];
        } catch (\Exception $e) {
            $error = 'Opportunity Export Error, Order Id ' . $incrementId . ' Reason >>' . $e->getMessage();
            $this->_helperData->eshopsyncLog($error);
        }
    }

    public function createSalesforceOrderLine($thisOrder, &$reqArr)
    {
        $productsArray = [];
        try {
            $incrementId = $thisOrder->getIncrementId();
            $Orderid = $thisOrder->getId();
            $items = $thisOrder->getAllItems();
            foreach ($items as $k => $item) {
                $bundleProductCollectivePrice = $sforceProductId = 0;
                $pDAtSalesforce = $item->getName();
                $productId = $item->getProductId();
                /*check for other types of products*/
                $basePrice = $item->getPriceInclTax();
                $itemType = $item->getProductType();
                if ($itemType == 'configurable') {
                    $pDAtSalesforce = '';
                    $selectedAttributeInformation = $item->getProductOptions()['attributes_info'];
                    foreach ($selectedAttributeInformation as $value) {
                        $pDAtSalesforce .= $value['label'] . '-' . $value['value'] . '--';
                    }
                    $pDAtSalesforce = rtrim($pDAtSalesforce, '--');
                }
                $parentByChild = $this->_catalogProductTypeConfigurable->getParentIdsByChild($productId);
                if ($itemType == 'bundle') {
                    $p = $this->_catalogProduct->create()->load($productId);
                    $priceType = $p->getPriceType();
                    if (!$priceType) {
                        $basePrice = 0;
                    }
                    $pDAtSalesforce = '';
                    $selectedAttributeInformation = $item->getProductOptions()['bundle_options'];
                    foreach ($selectedAttributeInformation as $value) {
                        $pDAtSalesforce .= $value['value'][0]['title'] . '--' . $value['value'][0]['qty'] . '--' . $value['value'][0]['price'] . '-----';
                        $bundleProductCollectivePrice += $value['value'][0]['price'];
                    }
                    $pDAtSalesforce = rtrim($pDAtSalesforce, '--');
                    $basePrice = $bundleProductCollectivePrice;
                }
                if ($item->getParentItemId() != null) {
                    $parentId = $item->getParentItemId();
                    $parent = $this->_salesItem->create()->load($parentId);
                    if ($parent->getProductType() == 'configurable') {
                        $basePrice = $parent->getPriceInclTax();
                    }
                }
                /* Fetching Salesforce Product id */
                $data = $this->_helperData->fetchMappingDetails($this->_eshopsynProduct, $productId);
                if ($data) {
                    $sforceProductId = $data[0]['sforce_id'];
                } else {
                    $sforceProductId = $this->_helperProduct->syncSpecificProduct($productId);
                }

                $parentByChild = $this->_catalogProductTypeConfigurable->getParentIdsByChild($productId);
                if ($item->getParentItemId() != null) {
                    continue;
                }
                if ($sforceProductId) {
                    try {
                        $priceBook = $this->_helperData->getConfigValue('catPdt', 'pricebook');
                        $pricebook = $this->_json->jsonDecode($priceBook)['id'];
                        /* Need to fix this wastage of API hit */
                        $pricebookResult = $this->_helperConnection->getSObject("SELECT Id FROM PricebookEntry WHERE Product2Id = '" . $sforceProductId . "' and Pricebook2Id = '" . $pricebook . "'");
                        if ($pricebookResult) {
                            $orderItems = new \stdclass();
                            $itemID = $item->getItemId();
                            $orderItems->Quantity = $item->getQtyOrdered();
                            $orderItems->OrderId = '@{order_' . $Orderid . '.id}';
                            $orderItems->UnitPrice = $basePrice;
                            if ($itemType == 'bundle') {
                                $orderItems->webkul_es_mage__Grouped_Product_Description__c = $pDAtSalesforce;
                            } else {
                                $orderItems->Description = $pDAtSalesforce;
                            }
                            $orderItems->PricebookEntryId = $pricebookResult['Id'];
                            $reqArr[] = [
                                'method' => 'PATCH',
                                'referenceId' => 'lineItem_' . $productId . '_' . $k,
                                'url' => '/services/data/v46.0/sobjects/OrderItem/webkul_es_mage__Magento_Order_Item_ID__c/' . $itemID,
                                'body' => $orderItems,
                            ];
                        }
                    } catch (\Exception $e) {
                        $this->_helperData->eshopsyncLog($e->getMessage());
                    }
                }
            }
        } catch (\Exception $e) {
            $this->_helperData->eshopsyncLog($e->getMessage());
        }
    }

    public function createSalesforceOpportunityLine($thisOrder, &$reqArr)
    {
        $productsArray = [];
        try {
            $incrementId = $thisOrder->getIncrementId();
            $Orderid = $thisOrder->getId();
            $items = $thisOrder->getAllItems();
            foreach ($items as $k => $item) {
                $bundleProductCollectivePrice = $sforceProductId = 0;
                $pDAtSalesforce = $item->getName();
                $productId = $item->getProductId();
                /*check for other types of products*/
                $basePrice = $item->getPriceInclTax();
                $itemType = $item->getProductType();
                if ($itemType == 'configurable') {
                    $pDAtSalesforce = '';
                    $selectedAttributeInformation = $item->getProductOptions()['attributes_info'];
                    foreach ($selectedAttributeInformation as $value) {
                        $pDAtSalesforce .= $value['label'] . '-' . $value['value'] . '--';
                    }
                    $pDAtSalesforce = rtrim($pDAtSalesforce, '--');
                }
                $parentByChild = $this->_catalogProductTypeConfigurable->getParentIdsByChild($productId);
                if ($itemType == 'bundle') {
                    $p = $this->_catalogProduct->create()->load($productId);
                    $priceType = $p->getPriceType();
                    if (!$priceType) {
                        $basePrice = 0;
                    }
                    $pDAtSalesforce = '';
                    $selectedAttributeInformation = $item->getProductOptions()['bundle_options'];
                    foreach ($selectedAttributeInformation as $value) {
                        $pDAtSalesforce .= $value['value'][0]['title'] . '--' . $value['value'][0]['qty'] . '--' . $value['value'][0]['price'] . '-----';
                        $bundleProductCollectivePrice += $value['value'][0]['price'];
                    }
                    $pDAtSalesforce = rtrim($pDAtSalesforce, '--');
                    $basePrice = $bundleProductCollectivePrice;
                }
                if ($item->getParentItemId() != null) {
                    $parentId = $item->getParentItemId();
                    $parent = $this->_salesItem->create()->load($parentId);
                    if ($parent->getProductType() == 'configurable') {
                        $basePrice = $parent->getPriceInclTax();
                    }
                }
                /* Fetching Salesforce Product id */
                $data = $this->_helperData->fetchMappingDetails($this->_eshopsynProduct, $productId);
                if ($data) {
                    $sforceProductId = $data[0]['sforce_id'];
                } else {
                    $sforceProductId = $this->_helperProduct->syncSpecificProduct($productId);
                }
                $parentByChild = $this->_catalogProductTypeConfigurable->getParentIdsByChild($productId);
                if (isset($parentByChild[0])) {
                    continue;
                }
                if ($item->getParentItemId() != null) {
                    continue;
                }

                if ($sforceProductId) {
                    try {

                        $priceBook = $this->_helperData->getConfigValue('catPdt', 'pricebook');
                        $pricebook = $this->_json->jsonDecode($priceBook)['id'];
                        /* Need to fix this wastage of API hit */
                        $pricebookResult = $this->_helperConnection->getSObject("SELECT Id FROM PricebookEntry WHERE Product2Id = '" . $sforceProductId . "' and Pricebook2Id = '" . $pricebook . "'");
                        if ($pricebookResult) {

                            $itemID = $item->getItemId();
                            $OppId = '@{opportunity_' . $Orderid . '.id}';

                            $productName = $this->_helperConnection->getSObject("SELECT Name FROM Product2 WHERE Id = '" . $sforceProductId . "'");

                            $name = $_SERVER['SERVER_NAME'] . '_' . $itemID . ' ' . $productName['Name'];

                            $OpportunityLineItemId = $this->_helperConnection->getSObject("SELECT Id FROM OpportunityLineItem WHERE Product2Id = '" . $sforceProductId . "' AND Name ='" . $name . "'");

                            $oppItems = new \stdclass();

                            $oppItems->Quantity = $item->getQtyOrdered();
                            $oppItems->OpportunityId = $OppId;
                            $oppItems->UnitPrice = $basePrice;

                            if ($itemType == 'bundle') {
                                $oppItems->Description = $pDAtSalesforce;
                            } else {
                                $oppItems->Description = $pDAtSalesforce;
                            }
                            $oppItems->PricebookEntryId = $pricebookResult['Id'];

                            if (count($OpportunityLineItemId) > 0) {
                                $reqArr[] = [
                                    'method' => 'PATCH',
                                    'referenceId' => 'opportunityLineItem_' . $productId . '_' . $k,
                                    'url' => '/services/data/v46.0/sobjects/OpportunityLineItem/Id/' . $OpportunityLineItemId['Id'],
                                    'body' => $oppItems,
                                ];
                            } else {

                                $reqArr[] = [
                                    'method' => 'POST',
                                    'referenceId' => 'opportunityLineItem_' . $productId . '_' . $k,
                                    'url' => '/services/data/v46.0/sobjects/OpportunityLineItem',
                                    'body' => $oppItems,
                                ];
                            }
                        }
                    } catch (\Exception $e) {
                        $this->_helperData->eshopsyncLog($e->getMessage());
                    }
                }
            }
        } catch (\Exception $e) {
            $this->_helperData->eshopsyncLog($e->getMessage());
        }
    }

    public function deleteMapping($magento_id)
    {
        $model = $this->_eshopsyncOrder->create();
        $collection = $model->getCollection()->addFieldToFilter('magento_id', $magento_id)->getData();
        if (count($collection) > 0) {
            foreach ($collection as $row) {
                $model->setId($row['entity_id'])->delete();
            }
        }
        $model = $this->_eshopsyncOpportunity->create();
        $collection = $model->getCollection()->addFieldToFilter('magento_id', $magento_id)->getData();
        if (count($collection) > 0) {
            foreach ($collection as $row) {
                $model->setId($row['entity_id'])->delete();
            }
        }
    }
}
